@@ -207,12 +207,14 @@ class BaseRegridder(object):
         weights=None,
         ignore_degenerate=None,
         input_dims=None,
+        unmapped_to_nan=False,
     ):
         """
         Base xESMF regridding class supporting ESMF objects: `Grid`, `Mesh` and `LocStream`.
 
         Create or use existing subclasses to support other types of input objects. See for example `Regridder`
-        to regrid `xarray.DataArray` objects, or `SpatialAverager` to average grids over regions defined by polygons.
+        to regrid `xarray.DataArray` objects, or `SpatialAverager`
+        to average grids over regions defined by polygons.
 
         Parameters
         ----------
@@ -271,6 +273,12 @@ class BaseRegridder(object):
             If not given or if those are not found on the regridded object, regridding
             uses the two last dimensions of the object (or the last one for input LocStreams and Meshes).
 
+        unmapped_to_nan: boolean, optional
+            Set values of unmapped points to `np.nan` instead of zero (ESMF default). This is useful for
+            target cells lying outside of the source domain when no output mask is defined.
+            If an output mask is defined, or regridding method is `nearest_s2d` or `nearest_d2s`,
+            this option has no effect.
+
         Returns
         -------
         baseregridder : xESMF BaseRegridder object
@@ -313,8 +321,10 @@ class BaseRegridder(object):
         # Convert weights, whatever their format, to a sparse coo matrix
         self.weights = read_weights(weights, self.n_in, self.n_out)
 
-        # replace zeros by NaN in mask
-        if self.grid_out.mask is not None and self.grid_out.mask[0] is not None:
+        # replace zeros by NaN for weight matrix entries of unmapped target cells if specified or a mask is present
+        if (
+            (self.grid_out.mask is not None) and (self.grid_out.mask[0] is not None)
+        ) or unmapped_to_nan is True:
             self.weights = add_nans_to_weights(self.weights)
 
         # follows legacy logic of writing weights if filename is provided
@@ -369,7 +379,7 @@ class BaseRegridder(object):
         esmf_regrid_finalize(regrid)  # only need weights, not regrid object
         return w
 
-    def __call__(self, indata, keep_attrs=False):
+    def __call__(self, indata, keep_attrs=False, skipna=False, na_thres=1.0):
         """
         Apply regridding to input data.
 
@@ -394,6 +404,27 @@ class BaseRegridder(object):
             Keep attributes for xarray DataArrays or Datasets.
             Defaults to False.
 
+        skipna: bool, optional
+            Whether to skip missing values when regridding.
+            When set to False, an output value is masked when a single
+            input value is missing and no grid mask is provided.
+            When set to True, missing values do not contaminate the regridding
+            since only valid values are taken into account.
+            In this case, a given output point is set to NaN only if the ratio
+            of missing values exceeds the level set by `na_thres`:
+            for instance, when the center of a cell is computed linearly
+            from its four corners, one of which is missing, the output value
+            is set to NaN if `na_thres` is greater or equal to 0.25.
+
+        na_thres: float, optional
+            A value within the [0., 1.] interval that defines the maximum
+            ratio of missing grid points involved in the regrdding over which
+            the output value is set to NaN. For instance, if `na_thres` is set
+            to 0, the output value is NaN if a single NaN is found in the input
+            values that are used to compute the output value; similarly,
+            if `na_thres` is set to 1, all input values must be missing to
+            mask the output value.
+
         Returns
         -------
         outdata : Data type is the same as input data type.
@@ -409,23 +440,43 @@ class BaseRegridder(object):
 
         """
         if isinstance(indata, np.ndarray):
-            return self.regrid_numpy(indata)
+            return self.regrid_numpy(indata, skipna=skipna, na_thres=na_thres)
         elif isinstance(indata, dask_array_type):
-            return self.regrid_dask(indata)
+            return self.regrid_dask(indata, skipna=skipna, na_thres=na_thres)
         elif isinstance(indata, xr.DataArray):
-            return self.regrid_dataarray(indata, keep_attrs=keep_attrs)
-        elif isinstance(indata, xr.Dataset):
-            return self.regrid_dataset(indata, keep_attrs=keep_attrs)
-        else:
-            raise TypeError(
-                'input must be numpy array, dask array, ' 'xarray DataArray or Dataset!'
+            return self.regrid_dataarray(
+                indata, keep_attrs=keep_attrs, skipna=skipna, na_thres=na_thres
             )
+        elif isinstance(indata, xr.Dataset):
+            return self.regrid_dataset(
+                indata, keep_attrs=keep_attrs, skipna=skipna, na_thres=na_thres
+            )
+        else:
+            raise TypeError('input must be numpy array, dask array, xarray DataArray or Dataset!')
 
     @staticmethod
-    def _regrid_array(indata, *, weights, shape_in, shape_out, sequence_in):
+    def _regrid_array(indata, *, weights, shape_in, shape_out, sequence_in, skipna, na_thres):
+
         if sequence_in:
             indata = np.expand_dims(indata, axis=-2)
-        return apply_weights(weights, indata, shape_in, shape_out)
+
+        # skipna: set missing values to zero
+        if skipna:
+            missing = np.isnan(indata)
+            indata = np.where(missing, 0.0, indata)
+
+        # apply weights
+        outdata = apply_weights(weights, indata, shape_in, shape_out)
+
+        # skipna: Compute the influence of missing data at each interpolation point and filter those not meeting acceptable threshold.
+        if skipna:
+            fraction_valid = apply_weights(weights, (~missing).astype('d'), shape_in, shape_out)
+            tol = 1e-6
+            bad = fraction_valid < np.clip(1 - na_thres, tol, 1 - tol)
+            fraction_valid[bad] = 1
+            outdata = np.where(bad, np.nan, outdata / fraction_valid)
+
+        return outdata
 
     @property
     def _regrid_kwargs(self):
@@ -436,12 +487,14 @@ class BaseRegridder(object):
             'shape_out': self.shape_out,
         }
 
-    def regrid_numpy(self, indata):
+    def regrid_numpy(self, indata, skipna=False, na_thres=1.0):
         """See __call__()."""
-        outdata = self._regrid_array(indata, **self._regrid_kwargs)
+        outdata = self._regrid_array(
+            indata, skipna=skipna, na_thres=na_thres, **self._regrid_kwargs
+        )
         return outdata
 
-    def regrid_dask(self, indata):
+    def regrid_dask(self, indata, skipna=False, na_thres=1.0):
         """See __call__()."""
 
         extra_chunk_shape = indata.chunksize[0:-2]
@@ -453,20 +506,23 @@ class BaseRegridder(object):
             indata,
             dtype=float,
             chunks=output_chunk_shape,
+            skipna=skipna,
+            na_thres=na_thres,
             **self._regrid_kwargs,
         )
 
         return outdata
 
-    def regrid_dataarray(self, dr_in, keep_attrs=False):
+    def regrid_dataarray(self, dr_in, keep_attrs=False, skipna=False, na_thres=1.0):
         """See __call__()."""
 
         input_horiz_dims, temp_horiz_dims = self._parse_xrinput(dr_in)
-
+        kwargs = self._regrid_kwargs.copy()
+        kwargs.update(skipna=skipna, na_thres=na_thres)
         dr_out = xr.apply_ufunc(
             self._regrid_array,
             dr_in,
-            kwargs=self._regrid_kwargs,
+            kwargs=kwargs,
             input_core_dims=[input_horiz_dims],
             output_core_dims=[temp_horiz_dims],
             dask='parallelized',
@@ -480,12 +536,15 @@ class BaseRegridder(object):
 
         return self._format_xroutput(dr_out, temp_horiz_dims)
 
-    def regrid_dataset(self, ds_in, keep_attrs=False):
+    def regrid_dataset(self, ds_in, keep_attrs=False, skipna=False, na_thres=1.0):
         """See __call__()."""
 
         # get the first data variable to infer input_core_dims
         input_horiz_dims, temp_horiz_dims = self._parse_xrinput(ds_in)
 
+        kwargs = self._regrid_kwargs.copy()
+        kwargs.update(skipna=skipna, na_thres=na_thres)
+
         non_regriddable = [
             name
             for name, data in ds_in.data_vars.items()
@@ -496,7 +555,7 @@ class BaseRegridder(object):
         ds_out = xr.apply_ufunc(
             self._regrid_array,
             ds_in,
-            kwargs=self._regrid_kwargs,
+            kwargs=kwargs,
             input_core_dims=[input_horiz_dims],
             output_core_dims=[temp_horiz_dims],
             dask='parallelized',
@@ -668,6 +727,12 @@ class Regridder(BaseRegridder):
             If False (default), raise error if grids contain degenerated cells
             (i.e. triangles or lines, instead of quadrilaterals)
 
+        unmapped_to_nan: boolean, optional
+            Set values of unmapped points to `np.nan` instead of zero (ESMF default). This is useful for
+            target cells lying outside of the source domain when no output mask is defined.
+            If an output mask is defined, or regridding method is `nearest_s2d` or `nearest_d2s`,
+            this option has no effect.
+
         Returns
         -------
         regridder : xESMF regridder object
@@ -885,6 +950,7 @@ class SpatialAverager(BaseRegridder):
             filename=filename,
             reuse_weights=reuse_weights,
             ignore_degenerate=ignore_degenerate,
+            unmapped_to_nan=False,
         )
 
     def _compute_weights(self):
@@ -899,12 +965,20 @@ class SpatialAverager(BaseRegridder):
         # Get weights for single polygons and holes
         # Stack everything together
         reg_ext = BaseRegridder(
-            mesh_ext, self.grid_in, 'conservative', ignore_degenerate=self.ignore_degenerate
+            mesh_ext,
+            self.grid_in,
+            'conservative',
+            ignore_degenerate=self.ignore_degenerate,
+            unmapped_to_nan=False,
         )
         if len(holes) > 0 and not self.ignore_holes:
             mesh_holes, shape_holes = polys_to_ESMFmesh(holes)
             reg_holes = BaseRegridder(
-                mesh_holes, self.grid_in, 'conservative', ignore_degenerate=self.ignore_degenerate
+                mesh_holes,
+                self.grid_in,
+                'conservative',
+                ignore_degenerate=self.ignore_degenerate,
+                unmapped_to_nan=False,
             )
             w_all = sps.hstack((reg_ext.weights.tocsc(), -reg_holes.weights.tocsc()))
         else:
